@@ -160,11 +160,11 @@ sub new {
 			domain2proxy => $d2p,
 		);
 	}
-	push @$ua_cleanup, [ 
+	push @$ua_cleanup, [
 		sub {
 			my ($self,$legs) = @_;
 			$self->{dispatcher}->remove_leg(@$legs);
-		}, 
+		},
 		$self,$legs
 	] if @$legs;
 
@@ -300,9 +300,13 @@ sub register {
 
 	my $from = delete $args{from} || $self->{from}
 		|| croak( "unknown from" );
-	my $contact = delete $args{contact} || $self->{contact} || $from;
-	my $local = $leg->{addr}.':'.$leg->{port};
-	$contact.= '@'.$local unless $contact =~s{\@([\w\-\.:]+)}{\@$local};
+
+	my $contact = delete $args{contact} || $self->{contact};
+	if ( ! $contact) {
+		$contact = $from;
+		my $local = $leg->{addr}.':'.$leg->{port};
+		$contact.= '@'.$local unless $contact =~s{\@([\w\-\.:]+)}{\@$local};
+	}
 
 	my %rarg = (
 		from => $from,
@@ -319,14 +323,17 @@ sub register {
 	my $cb = sub {
 		my ($self,$cb_final,$expires,$endpoint,$ctx,$errno,$code,$packet,$leg,$from) = @_;
 		if ( $code && $code =~m{^2\d\d} ) {
-			my $exp = $packet->get_header( 'Expires' );
-			if ( ! defined $exp ) {
-				foreach my $c ( $packet->get_header( 'contact' ) ) {
-					my ($addr,$p) = sip_hdrval2parts( contact => $c );
-					defined( my $e = $p->{expires} ) || next;
-					$exp = $e if ! defined($exp) || $e < $exp;
-				}
+			# use expires info on contact
+			# if none given use global expires header
+			# see rfc3261 10.3.8,10.2.4
+			my $exp;
+			for my $c ( $packet->get_header( 'contact' ) ) {
+				my ($addr,$p) = sip_hdrval2parts( contact => $c );
+				defined( my $e = $p->{expires} ) or next;
+				sip_uri_eq($addr,$contact) or next; # not me
+				$exp = $e if ! defined($exp) || $e < $exp;
 			}
+			$exp = $packet->get_header( 'Expires' ) if ! defined $exp;
 			$$expires = $exp;
 			invoke_callback( $cb_final, 'OK', expires => $exp );
 
@@ -384,7 +391,7 @@ sub invite {
 #      if sub and sub returns 1 call gets accepted, if sub returns 0 it gets rejected
 #    cb_create: optional callback called on creation of newly created
 #      Net::SIP::Simple::Call. If returns false the call will be closed.
-#      If returns a callback (e.g some ref) it will be used instead of 
+#      If returns a callback (e.g some ref) it will be used instead of
 #      Net::SIP::Simple::Call to handle the data
 #    cb_established: callback called after receiving ACK
 #    cb_cleanup: called on destroy of call object
@@ -467,7 +474,7 @@ sub create_registrar {
 }
 
 ###########################################################################
-# setup a stateless proxy 
+# setup a stateless proxy
 # Args: ($self,%args)
 #   %args: see Net::SIP::StatelessProxy
 # Returns: $proxy
@@ -485,7 +492,7 @@ sub create_stateless_proxy {
 ###########################################################################
 # setup chain of handlers, e.g. first authorize all requests, everything
 # else gets handled by stateless proxy etc
-# Args: ($self,$objects,%args) 
+# Args: ($self,$objects,%args)
 # Returns: $chain
 ###########################################################################
 sub create_chain {
